mod accel;
mod config;
mod gps;
mod modem;
#[allow(dead_code)]
mod command;

use anyhow;
use std::thread::{self, JoinHandle};
use std::cell::RefCell;
use std::time::Duration;

use esp_idf_hal::peripherals::Peripherals;

thread_local! {
    static TLS: RefCell<u32> = RefCell::new(13);
}


fn main() -> anyhow::Result<()> {
    esp_idf_sys::link_patches();

    let dp = Peripherals::take().expect("error taking peripherals");

    println!("Rust main thread: {:?}", thread::current());

    // LilyGo TTGO T-Call sim800l board serial pins.
    let modem_rx = dp.pins.gpio26;
    let modem_tx = dp.pins.gpio27;
    // LilyGo TTGO T-Call sim800l board power / reset pins.
    let modem_pwrkey = dp.pins.gpio4.into_output()?;
    let modem_rst = dp.pins.gpio5.into_output()?;
    let modem_power = dp.pins.gpio23.into_output()?;
    // UART interface for the GSM modem
    let modem_uart = dp.uart1;

    let mut threads: Vec<JoinHandle<anyhow::Result<_>>> = vec![];

    println!("Rust main thread: {:?}", thread::current());


    TLS.with(|tls| {
        println!("Main TLS before change: {}", *tls.borrow());
    });

    TLS.with(|tls| *tls.borrow_mut() = 42);

    TLS.with(|tls| {
        println!("Main TLS after change: {}", *tls.borrow());
    });

    let (gps_sender, receiver) = std::sync::mpsc::sync_channel::<modem::Msg>(1);

    let accel_sender = gps_sender.clone();

    threads.push(thread::spawn(move || gps::main(gps_sender)));
    thread::sleep(Duration::from_millis(1000));
    threads.push(thread::spawn(move || accel::main(accel_sender)));
    thread::sleep(Duration::from_millis(1000));

    let _ = modem::main(modem_rx, modem_tx, modem_uart, modem_pwrkey, modem_rst, modem_power, receiver)?;

    Ok(())
}
